/********************************************************************** 
 * Copyright (c) 2005 IBM Corporation and others. 
 * All rights reserved.   This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at 
 * http://www.eclipse.org/legal/epl-v10.html 
 * $Id: FileServerCommandFactory.java,v 1.9 2005/06/17 19:43:31 sschneid Exp $ 
 * 
 * Contributors: 
 * IBM - Initial API and implementation 
 **********************************************************************/

package org.eclipse.hyades.internal.execution.core.file;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.BindException;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketException;
import java.nio.channels.SocketChannel;
import java.util.HashMap;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.hyades.execution.core.file.IFileLocation;
import org.eclipse.hyades.execution.core.file.IFileManagerExtended.Cookie;
import org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList;
import org.eclipse.hyades.execution.core.file.IFileManagerExtended.Option;

/**
 * Used by the client and server side of the file manager implementation to
 * create commands that encapsulate the state and behavior associated with a
 * particular action that should be conducted.
 * <p>
 * There is a maintained singleton on the server and a qualified set of
 * instances on the client side per address and port combination. The current
 * supports commands are gathered from the public interface to the file server
 * command factory.
 * <p>
 * This class encapsulates the client side connection to the server, figuring
 * out the proper port to use to initiate file server commands and whether to
 * create new or reuse existing sockets. In server operation, this factory will
 * operate on the socket channels that are passed-in per call and not keep the
 * server running -- this is done in the file server extended class.
 * <p>
 * There is also a polymorophic create method that will create a command based
 * on the fully-qualified Java class name; this is used on the server-side to
 * create commands based on the comman identity that is sent over the wire.
 * 
 * @see org.eclipse.hyades.internal.execution.file.FileServerExtended
 * @see org.eclipse.hyades.execution.local.file.FileManagerExtendedImpl
 * @see #createDeleteFileCommand(Cookie, IProgressMonitor)
 * @see #createDeleteFileCommand(FileIdentifierList, IProgressMonitor)
 * @see #createGetFileCommand(Cookie, FileIdentifierList, FileIdentifierList,
 *      Option[], IProgressMonitor)
 * @see #createListContentCommand(FileIdentifierList, IProgressMonitor)
 * @see #createModifyPermissionCommand(FileIdentifierList, String,
 *      IProgressMonitor)
 * @see #createPutFileCommand(Cookie, FileIdentifierList, FileIdentifierList,
 *      Option[], IProgressMonitor)
 * @see #createQueryServerStatusCommand()
 * @author Scott E. Schneider
 */
public final class FileServerCommandFactory implements
        IFileServerCommandFactory {

    /**
     * Factor to multiply the previous timeout by each time a new attempt is
     * required
     */
    private static final float DEFAULT_CONNECT_RETRY_CALCULATION_FACTOR = 0.90f;

    /**
     * When the retry connect timeout drops below this cutoff, the connection
     * algorithm gives up and throws a server not available exception
     */
    private static final int DEFAULT_CONNECT_RETRY_CUTOFF = 2000;

    /**
     * Used to recalculate the timeout value in the recalculate method in this
     * class
     */
    private static final int DEFAULT_CONNECT_RETRY_INITIAL_TIMEOUT = 3000;

    /**
     * The offset value is added to all recalculated values before use,
     * including the initial value
     */
    private static final int DEFAULT_CONNECT_RETRY_OFFSET = 200;

    /**
     * Default port to connect on
     */
    private static final int DEFAULT_PORT = 10005;

    /**
     * Socket timeout to use by default
     */
    private static final int DEFAULT_SOCKET_TIMEOUT = 10000;

    /**
     * Stores the factories, used on the client-side
     */
    private static final HashMap factories;

    /**
     * Stores the one factory, used on the server-side
     */
    private static final IFileServerCommandFactory factory;

    /**
     * Create the pool of factories to use
     */
    static {
        factories = new HashMap();
        factory = new FileServerCommandFactory();
    }

    /**
     * Singleton instance used by file server, no address is needed since its
     * the server
     * 
     * @return the factory to interpet commands and create concrete commands to
     *         execute
     */
    public synchronized static IFileServerCommandFactory getInstance() {
        return FileServerCommandFactory.factory;
    }

    /**
     * A pool of factories is kept, qualified by the address of the server
     * 
     * @param fileServerLocation
     *            the location and other important information qualifying the
     *            exact file server host to use
     * @return a factory ready to use for the specified address
     */
    public synchronized static IFileServerCommandFactory getInstance(
            IFileLocation fileServerLocation) {

        // Retrieve internet address and port from the location instance
        InetAddress address = fileServerLocation.getInetAddress();
        int port = fileServerLocation.getPort();
        InetSocketAddress fileServerAddress = new InetSocketAddress(address,
                port);

        // Determine if a factory already exists for this address
        IFileServerCommandFactory factory = (IFileServerCommandFactory) FileServerCommandFactory.factories
                .get(fileServerAddress);

        // If a factory doesn't already exist, create one and store it
        if (factory == null) {
            factory = new FileServerCommandFactory(fileServerAddress);
            FileServerCommandFactory.factories.put(fileServerAddress, factory);
        }

        // Return a fully-initialized factory
        return factory;

    }

    /**
     * Address to use for this file server command factory when creating
     * commands
     */
    private InetSocketAddress address;

    /**
     * A singleton, not instantiated outside of this class, used on the
     * server-side only
     */
    private FileServerCommandFactory() {
    }

    /**
     * A singleton, not instantiated outside of this class, used on
     * the client-side only
     * 
     * @param address the address to use
     */
    private FileServerCommandFactory(InetSocketAddress address) {
        this.address = address;
    }
    
    /**
     * Connect the socket channel using the initial plus offset timeout value
     * for exception conditions.
     * 
     * @return the socket channel to the server ready to use
     */
    private synchronized ISocketChannel connectSocketChannel() {
        return this
                .connectSocketChannel(FileServerCommandFactory.DEFAULT_CONNECT_RETRY_INITIAL_TIMEOUT
                        + FileServerCommandFactory.DEFAULT_CONNECT_RETRY_OFFSET);
    }

    /**
     * Create a socket channel to the server given the specified timeout value,
     * this timeout value will be used in exceptional conditions, and then
     * recalculated using the appropriately named method in this class
     * 
     * @param timeout
     *            the timeout value to use
     * @return the socket channel ready to use, should never be null
     * @throws a
     *             server not available exception if a socket cannot be
     *             connected
     * 
     * @see #recalculateTimeout(int)
     * @see ServerNotAvailableException
     */
    private synchronized ISocketChannel connectSocketChannel(int timeout) {

        // Allocate a socket channel to use
        SocketChannel socketChannel = null;

        try {

            // Attempt to open connection to file server using channels
            socketChannel = SocketChannel.open();
            socketChannel.configureBlocking(true);
            socketChannel.socket().setSoTimeout(FileServerCommandFactory.DEFAULT_SOCKET_TIMEOUT);
            socketChannel.socket().setTcpNoDelay(true);
            socketChannel.socket().setReuseAddress(true);
            socketChannel.connect(this.address);

            
            return SocketChannelFactory.getInstance().create(socketChannel);

        } catch (SocketException e1) {
            
            // If any bind exception or all connection exceptions except refuse
            if (e1 instanceof BindException || e1 instanceof ConnectException) {
            

                // Return immediately on a connection refused exception
                if (e1.getMessage().indexOf("refuse") != -1) {
                    throw new ServerNotAvailableException(e1);
                }
                
	            // Connection failed due to bind exception, try again after a calculated wait
	            try {
	                
	                if (timeout < FileServerCommandFactory.DEFAULT_CONNECT_RETRY_CUTOFF) {
	                    throw new ServerNotAvailableException(e1);
	                } else {
	                    
	                    // Wait the variable timeout amount before re-attempt
	                    this.wait(timeout);
	
	                    /*
	                     * Recurse back into this method with a shorter timeout
	                     * value, the recursion exits when timeout drops below
	                     * default timeout cutoff value and then a server not
	                     * available exception is thrown
	                     */
	                    return this.connectSocketChannel(this.recalculateTimeout(timeout));
	
	                }
	                
	            } catch (InterruptedException e2) {
	                throw new ServerNotAvailableException(e2);
	            }
	            
            }
            
            throw new ServerNotAvailableException(e1);

        } catch (Throwable t) {
            throw new ServerNotAvailableException(t);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createDeleteFileCommand(
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.Cookie,
     *      org.eclipse.core.runtime.IProgressMonitor)
     */
    public IDeleteFileCommand createDeleteFileCommand(Cookie cookie,
            IProgressMonitor monitor) {
        return new DeleteFileCommand(cookie, monitor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createDeleteFileCommand(
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList,
     *      org.eclipse.core.runtime.IProgressMonitor)
     */
    public IDeleteFileCommand createDeleteFileCommand(
            FileIdentifierList remoteIdentifiers, IProgressMonitor monitor) {
        return new DeleteFileCommand(this.connectSocketChannel(),
                remoteIdentifiers, monitor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createFileServerCommand(java.lang.String,
     *      org.eclipse.hyades.internal.execution.core.file.ISocketChannel)
     */
    public IFileServerCommand createFileServerCommand(String identity,
            ISocketChannel clientChannel)
            throws InvalidFileServerCommandException {

        try {

            // Dynamically instantiate class with given identity
            Class classObject = Class.forName(identity);

            // Invoke a non-default constructor passing in channel
            Constructor constructor = classObject
                    .getConstructor(new Class[] { ISocketChannel.class });
            IFileServerCommand command = (IFileServerCommand) constructor
                    .newInstance(new Object[] { clientChannel });

            // Return the command that is ready for requests
            return command;

        } catch (ClassNotFoundException e) {

            // Command could not be instantiated with the given identity
            e.printStackTrace();

            throw new InvalidFileServerCommandException();

        } catch (InstantiationException e) {

            // Command could not be instantiated with the given identity
            e.printStackTrace();

            throw new InvalidFileServerCommandException();

        } catch (IllegalAccessException e) {

            // Command could not be instantiated with the given identity
            e.printStackTrace();

            throw new InvalidFileServerCommandException();

        } catch (InvocationTargetException e) {

            // Command could not be instantiated with the given identity
            e.printStackTrace();

            throw new InvalidFileServerCommandException();

        } catch (NoSuchMethodException e) {

            // Command could not be instantiated with the given identity
            e.printStackTrace();

            throw new InvalidFileServerCommandException();

        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createGetFileCommand(
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList,
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList,
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.Option[],
     *      org.eclipse.core.runtime.IProgressMonitor)
     */
    public IGetFileCommand createGetFileCommand(
            FileIdentifierList localIdentifiers,
            FileIdentifierList remoteIdentifiers, Option[] options,
            IProgressMonitor monitor) {
        return new GetFileCommand(this.connectSocketChannel(),
                localIdentifiers, remoteIdentifiers, options, monitor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createListContentCommand(
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList,
     *      org.eclipse.core.runtime.IProgressMonitor)
     */
    public IListContentCommand createListContentCommand(
            FileIdentifierList remoteIdentifiers, IProgressMonitor monitor) {
        return new ListContentCommand(remoteIdentifiers, monitor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createModifyPermissionCommand(
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList,
     *      java.lang.String, org.eclipse.core.runtime.IProgressMonitor)
     */
    public IModifyPermissionCommand createModifyPermissionCommand(
            FileIdentifierList remoteIdentifiers, String permissionDirective,
            IProgressMonitor monitor) {
        return new ModifyPermissionCommand(remoteIdentifiers,
                permissionDirective, monitor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createPutFileCommand(
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.Cookie,
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList,
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList,
     *      org.eclipse.hyades.execution.core.file.IFileManagerExtended.Option[],
     *      org.eclipse.core.runtime.IProgressMonitor)
     */
    public IPutFileCommand createPutFileCommand(Cookie cookie,
            FileIdentifierList localIdentifiers,
            FileIdentifierList remoteIdentifiers, Option[] options,
            IProgressMonitor monitor) {
        return new PutFileCommand(this.connectSocketChannel(), cookie,
                localIdentifiers, remoteIdentifiers, options, monitor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.hyades.internal.execution.core.file.IFileServerCommandFactory#createQueryServerStatusCommand()
     */
    public IQueryServerStatusCommand createQueryServerStatusCommand() {
        return new QueryServerStatusCommand(Cookie.NONE, this
                .connectSocketChannel());
    }

    /**
     * Recalculate a new timeout value to try for the next connect attempt, used
     * typically when a bind exception occurs because an address is already in
     * use, this is due to local addresses being exhausted without old ones
     * being reclaimed yet, this is a temporary situation which we'll wait out
     * with this timeout value.
     * 
     * @param timeout
     *            the timeout value to seed the new timeout value on
     * @return the new timeout value that has been recalculated
     */
    private int recalculateTimeout(int timeout) {
        float recalculatedTimeout = timeout
                * FileServerCommandFactory.DEFAULT_CONNECT_RETRY_CALCULATION_FACTOR
                + FileServerCommandFactory.DEFAULT_CONNECT_RETRY_OFFSET;
        return Math.round(recalculatedTimeout);
    }

}