/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: Component.java,v 1.28 2005/03/23 08:09:06 dnsmith Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.logging.adapter.impl;
import org.eclipse.hyades.logging.adapter.AdapterInvalidConfig;
import org.eclipse.hyades.logging.adapter.IComponent;
import org.eclipse.hyades.logging.adapter.IProcessUnit;
import org.eclipse.hyades.logging.adapter.util.Messages;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.CompletionException;
import org.eclipse.hyades.logging.events.cbe.ComponentIdentification;
import org.eclipse.hyades.logging.events.cbe.EventFactory;
import org.eclipse.hyades.logging.events.cbe.EventFactoryHome;
import org.eclipse.hyades.logging.events.cbe.MsgDataElement;
import org.eclipse.hyades.logging.events.cbe.Situation;
import org.eclipse.hyades.logging.events.cbe.impl.EventFactoryContext;
import org.w3c.dom.Element;
/**
 * Abstract class provided for the convinience. Contains some default 
 * implementations of the main functionality for the component
 * Role of the abstract class Component is to provide initial most common 
 * implementation of  basic functionality for 
 * component.  So the class provide implementation for the getter
 * and setter for the componentName, getter and setter 
 * for the targetName, stabs for the start, stop and update functions.
 * @version 0.10
 */
public class Component implements IComponent
{
	protected IProcessUnit logger = null;
	private short loggingLevel = CommonBaseEvent.SEVERITY_UNKNOWN;
	private String name = null;
	private String role = null;
	private String executableClassName = null;
	private String description = null;
	private String uniqueID = null;
	// Version info
	private String roleVersion = null;
	private String implVersion = null;
	private String roleVersionDescription = null;
	private String implVersionDescription = null;
	private String roleCreationDate = null;
	private String implCreationDate = null;
	private IComponent[] components = null;
	private IComponent parent=null;
	private EventFactory eventFactory = null;
	
	private final static short SEVERITY_TRACE = 5;
	
	protected Status status;
	
	/*
	 * indicates if context is stopping
	 */
	protected boolean stopping = false;
	/**
	 * contains config document fragment as a string 
	 */
	private Element configuration = null;
	/**
	 * contains context config document fragment as a string 
	 */
	private Element contextConfiguration = null;
	/**
	 * Constructor for Component.
	 */
	public Component()
	{
		super();
		status = new Status();
	}
	/**
	 * Initialize all the Component level properties from the context config for this
	 * component, and then ask all child components to do the same.
	 * @see com.ibm.acad.general.common.IComponent#init()
	 */
	public boolean init()
	{
		Element element = getContextConfiguration();
		// 
		if (element.hasAttribute(Messages.getString("HyadesGADescriptionAttributeName")))
			setDescription(element.getAttribute(Messages.getString("HyadesGADescriptionAttributeName")));
		if (element.hasAttribute(Messages.getString("HyadesGARoleAttributeName")))
			setRole(element.getAttribute(Messages.getString("HyadesGARoleAttributeName")));
		if (element.hasAttribute(Messages.getString("HyadesGALoggingLevelAttributeName")))
			setLoggingLevel(
				Short
					.valueOf((element.getAttribute(Messages.getString("HyadesGALoggingLevelAttributeName"))))
					.shortValue());
		// set the version info if provided
		if (element.hasAttribute(Messages.getString("HyadesGARoleVersionAttributeName")))
			setRoleVersion(element.getAttribute(Messages.getString("HyadesGARoleVersionAttributeName")));
		if (element.hasAttribute(Messages.getString("HyadesGARoleCreationDateAttributeName")))
			setRoleCreationDate(element.getAttribute(Messages.getString("HyadesGARoleCreationDateAttributeName")));
		if (element.hasAttribute(Messages.getString("HyadesGARoleVersionDescriptionAttributeName")))
			setRoleVersionDescription(
				element.getAttribute(Messages.getString("HyadesGARoleVersionDescriptionAttributeName")));
		if (element.hasAttribute(Messages.getString("HyadesGAImplementationVersionAttributeName")))
			setImplementationVersion(
				element.getAttribute(Messages.getString("HyadesGAImplementationVersionAttributeName")));
		if (element.hasAttribute(Messages.getString("HyadesGAImplemenationCreationDateAttributeName")))
			setImplemenationCreationDate(
				element.getAttribute(Messages.getString("HyadesGAImplemenationCreationDateAttributeName")));
		if (element.hasAttribute(Messages.getString("HyadesGAImplemenationVersionDescriptionAttributeName")))
			setImplementationVersionDescription(
				element.getAttribute(Messages.getString("HyadesGAImplemenationVersionDescriptionAttributeName")));
		IComponent[] components = getComponents();
		if (components != null){
			for (int i = 0; i < components.length; i++) {
				components[i].setLogger(getLogger());
				components[i].init();
			}
		}
		
		return true;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#start()
	 */
	public void start()
	{
		status.setActive(true);
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#stop(int)
	 */
	public void stop(int toStop)
	{
	}
	
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#stop()
	 */
	public void stop() {
		
		/* Tell our children to stop if we have any */
		if (components != null){
			for (int i = 0; i < components.length; i++) {
				components[i].stop();
			}
		}
		stopping = true;
		status.setActive(false);
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#update(Element)
	 */
	public void update(Element config) throws AdapterInvalidConfig
	{
		setConfiguration(config);
		update();
	}
	/**
	 * Set Component instance level properties based on the component configuration Element.
	 * @see org.eclipse.hyades.logging.adapter.IComponent#update()
	 */
	public void update() throws AdapterInvalidConfig
	{
		Element element = getConfiguration();
		status.setName(getName());
		setDescription(element.getAttribute(Messages.getString("HyadesGADescriptionAttributeName")));
		setUniqueID(element.getAttribute(Messages.getString("HyadesGAUniqueIDAttributeName")));
		IComponent[] components = getComponents();
		if (components != null)
			for (int i = 0; i < components.length; i++) {
				components[i].update();
				/* Log the fact we have finished configuring this context */
				CommonBaseEvent configureEvent = getEventFactory().createCommonBaseEvent();

				configureEvent.setMsg(Messages.getString("HyadesGAComponent_ConfigurationComplete_INFO_", components[i].getName()));
				configureEvent.setSeverity(CommonBaseEvent.SEVERITY_INFORMATION);
				Situation configSituation = getEventFactory().createSituation();
				configSituation.setConfigureSituation("INTERNAL","SUCCESSFUL");
				configureEvent.setSituation(configSituation);
			    log(configureEvent);
			}
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getContextConfiguration()
	 */
	public Element getContextConfiguration()
	{
		return contextConfiguration;
	}
	/**
	 * Sets the context config.
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setContextConfiguration(Element)
	 * @param config The configuration to set
	 */
	public void setContextConfiguration(Element newConfiguration)
	{
		contextConfiguration = newConfiguration;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getConfiguration()
	 */
	public Element getConfiguration()
	{
		return configuration;
	}
	/**
	 * Sets the config.
	 * @param config The configuration to set
	 */
	public void setConfiguration(Element newConfiguration)
	{
		configuration = newConfiguration;
	}
	// getters and setters for private members
	
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getLogger()
	 */
	public IProcessUnit getLogger() {
		
		return logger;
	}
	
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setLogger(IProcessUnit)
	 */
	public void setLogger(IProcessUnit logger) {
		this.logger=logger;
	}
	
	public EventFactory getEventFactory()
	{
		if (eventFactory == null){
		    
		    EventFactoryHome eventFactoryHome = EventFactoryContext.getInstance().getEventFactoryHome("org.eclipse.hyades.logging.adapter.internal.util.RuntimeEventXMLFileEventFactoryHomeImpl",this.getClass().getClassLoader());
		    
		    if(eventFactoryHome != null){
		        eventFactory = eventFactoryHome.getEventFactory("org.eclipse.hyades.logging.adapter.impl.Component");
		        eventFactory.setCompleteEvent(true);
		    }
		    else{
		        eventFactory = EventFactoryContext.getInstance().getSimpleEventFactoryHome().getAnonymousEventFactory();		        
		    }
		}
		
		return eventFactory;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getName()
	 */
	public short getLoggingLevel()
	{
		return loggingLevel;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setName(short)
	 */
	public void setLoggingLevel(short newLevel)
	{
		loggingLevel = newLevel;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getName()
	 */
	public String getName()
	{
		return name;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setName(java.lang.String)
	 */
	public void setName(String newName)
	{
		name = newName;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getRole()
	 */
	public String getRole()
	{
		return role;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setRole(java.lang.String)
	 */
	public void setRole(String newRole)
	{
		role = newRole;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getDescription()
	 */
	public String getDescription()
	{
		return description;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setDescription(java.lang.String)
	 */
	public void setDescription(String newDescription)
	{
		description = newDescription;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getUniqueID()
	 */
	public String getUniqueID()
	{
		return uniqueID;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setUniqueID(java.lang.String)
	 */
	public void setUniqueID(String newUniqueID)
	{
		uniqueID = newUniqueID;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getExecutableClassName()
	 */
	public String getExecutableClassName()
	{
		return executableClassName;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setExecutableClassName(java.lang.String)
	 */
	public void setExecutableClassName(String newExecutableClassName)
	{
		executableClassName = newExecutableClassName;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getRoleVersion()
	 */
	public String getRoleVersion()
	{
		return roleVersion;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setRoleVersion(java.lang.String)
	 */
	public void setRoleVersion(String newRoleVersion)
	{
		roleVersion = newRoleVersion;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getRoleCreationDate()
	 */
	public String getRoleCreationDate()
	{
		return roleCreationDate;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setRoleCreationDate(java.lang.String)
	 */
	public void setRoleCreationDate(String newCreationDate)
	{
		roleCreationDate = newCreationDate;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getRoleVersionDescription()
	 */
	public String getRoleVersionDescription()
	{
		return roleVersionDescription;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setRoleVersionDescription(java.lang.String)
	 */
	public void setRoleVersionDescription(String newRoleVersionDescription)
	{
		roleVersionDescription = newRoleVersionDescription;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getImplementationVersion()
	 */
	public String getImplementationVersion()
	{
		return implVersion;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setImplementationVersion(java.lang.String)
	 */
	public void setImplementationVersion(String newImplementationVersion)
	{
		implVersion = newImplementationVersion;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getImplementationCreationDate()
	 */
	public String getImplementationCreationDate()
	{
		return implCreationDate;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setImplemenationCreationDate(java.lang.String)
	 */
	public void setImplemenationCreationDate(String newImplementationCreationDate)
	{
		implCreationDate = newImplementationCreationDate;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getImplementationVersionDescription()
	 */
	public String getImplementationVersionDescription()
	{
		return implVersionDescription;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setImplementationVersionDescription(java.lang.String)
	 */
	public void setImplementationVersionDescription(String newImplementationVersionDescription)
	{
		implVersionDescription = newImplementationVersionDescription;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getComponents()
	 */
	public IComponent[] getComponents()
	{
		return components;
	}
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setComponents(org.eclipse.hyades.logging.adapter.IComponent[])
	 */
	public void setComponents(IComponent[] newComponents)
	{
		components = newComponents;
		for(int i=0; i<components.length; i++) {
			components[i].setParent(this);
			
		}
		
	}
	public void clean()
	{
		if (components == null) {
			return;
		}
		
		for (int i = 0; i < components.length; i++)
		{
			components[i] = null;
		}
		components = null;
	}

	/**
	 * Tracing method with one substitution variable.  
	 * <p>
	 * The message with the specified resource key and substutution value is logged in a Common Base Event with severity 5.
	 * 
	 * @param msgKey resource key for trace message string.
	 * @param sub substitution value - null for no value
	 */
	public void trace(String msgKey, String sub) {
	    if ((msgKey != null) && (loggingLevel == 0)){
	    	String msg;
	    	if (sub != null) {
	    		msg = Messages.getString(msgKey, sub);
	    	}
	    	else {
	    		msg = Messages.getString(msgKey);
	    	}
	    	
	    	trace(msg);
	    }
	}

	/**
	 * Tracing method with two substitution variables.  
	 * <p>
	 * The message with the specified resource key and substutution values is logged in a Common Base Event with severity 5.
	 * 
	 * @param msgKey resource key for trace message string.
	 * @param sub1 first substitution value
	 * @param sub2 second substitution value
	 */
	public void trace(String msgKey, String sub1, String sub2) {
	    if (msgKey != null && sub1 != null && sub2 != null && (loggingLevel == 0)){
	    	trace(Messages.getString(msgKey, sub1, sub2), null);
	    }
	}
	
	/**
	 * Tracing method with three substitution variables.  
	 * <p>
	 * The message with the specified resource key and substutution values is logged in a Common Base Event with severity 5.
	 * 
	 * @param msgKey resource key for trace message string.
	 * @param sub1 first substitution value
	 * @param sub2 second substitution value
	 * @param sub3 third substitution value
	 */
	public void trace(String msgKey, String sub1, String sub2, String sub3) {
	    if (msgKey != null && sub1 != null && sub2 != null && sub3 != null && (loggingLevel == 0)){
	    	trace(Messages.getString(msgKey, sub1, sub2, sub3), null);
	    }
	}
	
	/**
	 * Tracing method.  
	 * <p>
	 * The trace message is logged as a Common Base Event with severity 5.
	 * 
	 * @param msg trace message string.
	 */
	private void trace(String msg) {
	    
	    if (msg != null){
			/* Log the fact we have finished configuring this context */
			CommonBaseEvent traceEvent = getEventFactory().createCommonBaseEvent();

			traceEvent.setMsg(msg);
			traceEvent.setSeverity(SEVERITY_TRACE);
			Situation reportSituation = getEventFactory().createSituation();
			reportSituation.setReportSituation("INTERNAL","TRACE");
			traceEvent.setSituation(reportSituation);
		    log(traceEvent);	    	
	    }
	}
	
	/**
	 * Logging method.  
	 * <p>
	 * The Common Base Event is logged based on its severity.
	 * 
	 * @param event The Common Base Event to be logged.
	 */
	public void log(CommonBaseEvent event){
	    
	    if ((event != null) && (event.getSeverity() >= loggingLevel)){
			
	    	//Resolves the event's message from the MsgDataElement:
	    	String msg = event.getMsg();
	    	
			if(msg == null || msg.length() == 0){
				
				MsgDataElement msgDataElement = event.getMsgDataElement();
				
				if(msgDataElement != null){
					
					String msgCatalogId = msgDataElement.getMsgCatalogId();
					
					if(msgCatalogId != null){
						
						String[] msgCatalogTokens = msgDataElement.getMsgCatalogTokensAsStrings();
						
						if((msgCatalogTokens == null) || (msgCatalogTokens.length == 0)){
							event.setMsg(Messages.getString(msgCatalogId));
						}
						else{
							event.setMsg(Messages.getString(msgCatalogId,msgCatalogTokens));							
						}
					}
				}
			}
			
			if(!event.isComplete()){
			    
			    try {
			        event.complete();                    
			    } 
			    catch (CompletionException c) {
			    }
			}		
			
			ComponentIdentification sourceComponentId = event.getSourceComponentId();
			
			if(sourceComponentId != null){
			    
				sourceComponentId.setComponent(getName());
				sourceComponentId.setSubComponent(getUniqueID());
				sourceComponentId.setComponentIdType(getRole());
			}
			
			/* Check if the logger has been set before processing the event */
			IProcessUnit glaLogger = getLogger();
			if (glaLogger != null) {
				glaLogger.processEventItems(new CommonBaseEvent[]{event});
			}
	    }
	}

	/**
	 * Logging method.
	 * 
	 * @param event The Common Base Event to be logged.
	 * @param level The severity of the  Common Base Event to be logged.
	 * @deprecated Set the severity on the Common Base Event and use the <code>log(CommonBaseEvent event)</code> API.
	 */
	public void log(CommonBaseEvent event, short level){
		
	    if (event != null){

	        event.setSeverity(level);
			
			log(event);
		}
	}
	
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#setParent(org.eclipse.hyades.logging.adapter.IComponent)
	 */
	public void setParent(IComponent parent) {
		this.parent=parent;
		
	}
	
	/**
	 * @see org.eclipse.hyades.logging.adapter.IComponent#getParent()
	 */
	public IComponent getParent() {
		return parent;
	}
	/**
	 * @return Returns the stopping.
	 */
	public boolean isStopping() {
		return stopping;
	}
	/**
	 * @param stopping The stopping to set.
	 */
	public void setStopping(boolean stopping) {
		this.stopping = stopping;
	}
	
	/**
	 * @return Returns the itemsProcessedCount.
	 */
	public int getItemsProcessedCount() {
		return status.getItemsProcessedCount();
	}
	
	/**
	 * Increments the itemsProcessedCount.
	 */	
	public void incrementItemsProcessedCount() {
		status.setItemsProcessedCount(status.getItemsProcessedCount()+1);
	}
	
	/**
	 * Increments the itemsProcessedCount by the value passed in.
	 * @param n Amount to increment the count by.
	 */	
	public void incrementItemsProcessedCount(int n) {
		status.setItemsProcessedCount(status.getItemsProcessedCount()+n);
	}
	/**
	 * @return Returns the status.
	 */
	public Status getComponentStatus() {
		return status;
	}
}
