/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TestNavigatorActionGroup.java,v 1.18 2005/04/22 14:51:01 bjiang Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.navigator.action;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.hyades.test.ui.TestUIImages;
import org.eclipse.hyades.test.ui.TestUIPlugin;
import org.eclipse.hyades.test.ui.internal.model.ui.RemoveChildrenAction;
import org.eclipse.hyades.test.ui.internal.navigator.TestNavigator;
import org.eclipse.hyades.ui.extension.INavigatorContribution;
import org.eclipse.hyades.ui.internal.action.CollapseAllAction;
import org.eclipse.hyades.ui.internal.navigator.INavigator;
import org.eclipse.hyades.ui.internal.navigator.ITreeNavigator;
import org.eclipse.hyades.ui.internal.navigator.TreeNavigator;
import org.eclipse.hyades.ui.internal.navigator.action.FrameListActionGroup;
import org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup;
import org.eclipse.hyades.ui.internal.navigator.action.NavigatorSorterActionGroup;
import org.eclipse.hyades.ui.internal.navigator.action.ToggleLinkingAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkingSet;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionContext;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.AddBookmarkAction;
import org.eclipse.ui.actions.AddTaskAction;
import org.eclipse.ui.actions.CloseResourceAction;
import org.eclipse.ui.actions.DeleteResourceAction;
import org.eclipse.ui.actions.OpenResourceAction;
import org.eclipse.ui.actions.RefreshAction;
import org.eclipse.ui.actions.WorkingSetFilterActionGroup;
import org.eclipse.ui.dialogs.PropertyDialogAction;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.views.framelist.TreeFrame;
import org.eclipse.ui.views.navigator.ResourceNavigatorMessages;

/**
 * Group action for the Test Navigator.
 * 
 * <p>This implementation is based on the 
 * {@link org.eclipse.ui.views.navigator.MainActionGroup} class.
 * 
 * @author marcelop
 * @since 0.0.1
 */
public class TestNavigatorActionGroup 
extends NavigatorActionGroup
{
	private Clipboard clipboard;

	protected OpenEditorAction openAction;
	protected AddBookmarkAction addBookmarkAction;
	protected AddTaskAction addTaskAction;	
	protected PropertyDialogAction propertyDialogAction;
	protected CollapseAllAction collapseAllAction;
	protected ToggleLinkingAction toggleLinkingAction;
	protected ToggleFolderAction toggleFolderAction;
	protected ToggleEObjectChildrenAction toggleEObjectChildren;
	protected OpenReportAction openReportAction;
	protected GenerateAction generateAction;	
	protected RefreshAction refreshAction;
	protected OpenResourceAction openProjectAction;
	protected CloseResourceAction closeProjectAction;
	protected StopAction stopAction;

	protected DeleteResourceAction deleteResourceAction;
	protected RemoveChildrenAction deleteChildEObject;
	
	protected NavigatorSorterActionGroup sorterActionGroup;
	protected NewActionGroup newActionGroup;
	protected WorkingSetFilterActionGroup workingSetFilterActionGroup;
	protected FrameListActionGroup frameListActionGroup;
	protected RenameAction renameAction;
//	protected MoveAction moveAction;
	protected CopyAction copyAction;
	protected PasteAction pasteAction;
	
	/**
	 * Constructor for NavigatorActionGroup
	 * @param testNavigator
	 */
	public TestNavigatorActionGroup(TestNavigator testNavigator)
	{
		super(testNavigator);
	}
	
	/**
	 * @see org.eclipse.ui.actions.ActionGroup#dispose()
	 */
	public void dispose()
	{
		newActionGroup.dispose();
		workingSetFilterActionGroup.dispose();
		sorterActionGroup.dispose();
		frameListActionGroup.dispose();
		generateAction.dispose();
		stopAction.dispose();
		renameAction.dispose();
		copyAction.dispose();
		if (clipboard != null) {
			clipboard.dispose();
			clipboard = null;
		}
		super.dispose();
	}

	/**
	 * @see org.eclipse.ui.actions.ActionGroup#setContext(org.eclipse.ui.actions.ActionContext)
	 */
	public void setContext(ActionContext context)
	{
		super.setContext(context);
		
		sorterActionGroup.setContext(context);
		newActionGroup.setContext(context);
		workingSetFilterActionGroup.setContext(context);
		frameListActionGroup.setContext(context); 
	}

	/**
	 * @see org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup#createActions()
	 */
	protected void createActions()
	{
		super.createActions();
		Shell shell = getNavigator().getSite().getShell();
		clipboard = new Clipboard(shell.getDisplay());

		openAction = new OpenEditorAction((TestNavigator)getNavigator(), TestUIPlugin.getString("ACT_OPEN")); //$NON-NLS-1$
		addBookmarkAction = new AddBookmarkAction(shell);
		addTaskAction = new AddTaskAction(shell);
		propertyDialogAction = new PropertyDialogAction(shell, getNavigator().getViewer());	

		collapseAllAction = new CollapseAllAction(((ITreeNavigator)getNavigator()).getTreeViewer());
		collapseAllAction.useDefaultDecorators();

		toggleLinkingAction = new ToggleLinkingAction(getNavigator(),
		        ResourceNavigatorMessages.getString("ToggleLinkingAction.text"));  //$NON-NLS-1$
		toggleLinkingAction.setToolTipText(ResourceNavigatorMessages.getString("ToggleLinkingAction.toolTip"));  //$NON-NLS-1$
		toggleLinkingAction.setImageDescriptor(getImageDescriptor("elcl16/synced.gif")); //$NON-NLS-1$
		toggleLinkingAction.setHoverImageDescriptor(getImageDescriptor("elcl16/synced.gif")); //$NON-NLS-1$

		toggleFolderAction = new ToggleFolderAction((TestNavigator)getNavigator(), 
		        TestUIPlugin.getString("ACT_SHOW_FLD_TXT")); //$NON-NLS-1$
		toggleFolderAction.setToolTipText(TestUIPlugin.getString("ACT_SHOW_FLD_DSC")); //$NON-NLS-1$
		TestUIImages.INSTANCE.setImageDescriptors(toggleFolderAction, TestUIImages.ACT_SHOW_FOLDER);

		toggleEObjectChildren = new ToggleEObjectChildrenAction((TestNavigator)getNavigator(), 
		        TestUIPlugin.getString("ACT_SHOW_EOBJ_CH_TXT")); //$NON-NLS-1$
		toggleEObjectChildren.setToolTipText(TestUIPlugin.getString("ACT_SHOW_EOBJ_CH_DSC")); //$NON-NLS-1$
		TestUIImages.INSTANCE.setImageDescriptors(toggleEObjectChildren, TestUIImages.ACT_SHOW_EOBJECT_CHILDREN);

		ISharedImages images = PlatformUI.getWorkbench().getSharedImages();
		deleteResourceAction = new DeleteResourceAction(shell);
		deleteResourceAction.setDisabledImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE_DISABLED));
		deleteResourceAction.setImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));		
		deleteResourceAction.setHoverImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE_HOVER));
		
		deleteChildEObject = new RemoveChildrenAction(true)
		{
			protected List saveResources(Set changedResources)
			{
				TestNavigator testNavigator = (TestNavigator)getNavigator();
				testNavigator.getTreeViewer().remove(getStructuredSelection().toArray());
				
				testNavigator.setMonitoringResourceChange(false);
				try
				{
					return super.saveResources(changedResources);
				}
				finally
				{
					testNavigator.setMonitoringResourceChange(true);
				}
			}
		};
		deleteChildEObject.setStructuredViewer(((TestNavigator)getNavigator()).getViewer());
		deleteChildEObject.setShowErrorDialog(true);
		
		openReportAction = new OpenReportAction();	
		generateAction = new GenerateAction();	
		refreshAction = new RefreshAction(shell);
		refreshAction.setDisabledImageDescriptor(getImageDescriptor("dlcl16/refresh_nav.gif"));//$NON-NLS-1$
		refreshAction.setImageDescriptor(getImageDescriptor("elcl16/refresh_nav.gif"));//$NON-NLS-1$	
		openProjectAction = new OpenResourceAction(shell);
		closeProjectAction = new CloseResourceAction(shell);
		stopAction = new StopAction();	
		renameAction = new RenameAction((TreeNavigator)getNavigator());
		
		pasteAction = new PasteAction(shell, clipboard);
		pasteAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));
		pasteAction.setDisabledImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_PASTE_DISABLED));
		copyAction = new CopyAction(clipboard);
		copyAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
		copyAction.setDisabledImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_COPY_DISABLED));		
	}

	/**
	 * @see org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup#createActionGroups()
	 */
	protected void createActionGroups()
	{
		super.createActionGroups();
		sorterActionGroup = new NavigatorSorterActionGroup(getNavigator());
		newActionGroup = new NewActionGroup((TestNavigator)getNavigator());
		
		IPropertyChangeListener workingSetUpdater = new IPropertyChangeListener()
		{
			public void propertyChange(PropertyChangeEvent event)
			{
				String property = event.getProperty();
				
				if (WorkingSetFilterActionGroup.CHANGE_WORKING_SET.equals(property))
				{
					INavigator navigator = getNavigator();
					Object newValue = event.getNewValue();
					
					if (newValue instanceof IWorkingSet)
					{	
						((TestNavigator)navigator).setWorkingSet((IWorkingSet) newValue);
					}
					else if (newValue == null)
					{
						((TestNavigator)navigator).setWorkingSet(null);
					}
				}
			}
		};
		workingSetFilterActionGroup = new WorkingSetFilterActionGroup(getNavigator().getViewer().getControl().getShell(), workingSetUpdater);
		workingSetFilterActionGroup.setWorkingSet(((TestNavigator)getNavigator()).getWorkingSet());
		frameListActionGroup = new FrameListActionGroup((TestNavigator)getNavigator());
	}

	/**
	 * @see org.eclipse.ui.actions.ActionGroup#fillContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void fillContextMenu(IMenuManager menu)
	{
		super.fillContextMenu(menu);
		boolean hasClosedProjects = false;
		boolean hasOpenProjects = false;
		boolean isProjectSelection = true;
		
		IStructuredSelection selection = (IStructuredSelection)getContext().getSelection();
		IStructuredSelection resSelection = getResourceSelection(selection);
		Iterator resources = selection.iterator();
		
		while (resources.hasNext() &&
				(!hasOpenProjects || !hasClosedProjects || isProjectSelection)) {
			Object next = resources.next();
			IProject project = null;
			
			if (next instanceof IProject) {
				project = (IProject) next;
			} else if (next instanceof IAdaptable) {
				project = (IProject) ((IAdaptable) next).getAdapter(IProject.class);
			}
			if (project == null) {
				isProjectSelection = false;
			} else {
				if (project.isOpen()) {
					hasOpenProjects = true;
				} else {
					hasClosedProjects = true;
				}
			}
		}	

		newActionGroup.fillContextMenu(menu);
		frameListActionGroup.fillContextMenu(menu);
		
		openAction.selectionChanged(selection);
		if(openAction.isApplicableForSelection())
		{
			menu.appendToGroup(INavigatorContribution.GROUP_OPEN, openAction);
//			menu.appendToGroup(INavigatorContribution.GROUP_OPEN, new Separator());			
		}
		
		
		if (!hasClosedProjects) {
			refreshAction.selectionChanged(selection);
			menu.appendToGroup(INavigatorContribution.GROUP_REFRESH, refreshAction);
		}
		
		if (isProjectSelection) {
			if (hasClosedProjects) {
				openProjectAction.selectionChanged(selection);
				menu.appendToGroup(INavigatorContribution.GROUP_REFRESH, openProjectAction);				
			}
			if (hasOpenProjects) {
				closeProjectAction.selectionChanged(selection);
				menu.appendToGroup(INavigatorContribution.GROUP_REFRESH, closeProjectAction);
			}
		}					
		
		generateAction.selectionChanged(selection);
		if(generateAction.isApplicableForSelection())
		{
			menu.appendToGroup(INavigatorContribution.GROUP_IMPORT, generateAction);
		}
	
		openReportAction.selectionChanged(selection);
		if(openReportAction.isApplicableForSelection())
		{
			menu.appendToGroup(INavigatorContribution.GROUP_IMPORT, openReportAction);
		}
		
		stopAction.selectionChanged(selection);
		if(stopAction.isApplicableForSelection())
		{
			menu.appendToGroup(INavigatorContribution.GROUP_DELETE, stopAction);
			stopAction.setEnabled(stopAction.isApplicableForEnablement());
		}

//		if(copyAction.updateSelection(selection))
		copyAction.selectionChanged(selection);
		if(copyAction.isApplicableForSelection())
		{
			menu.appendToGroup(INavigatorContribution.GROUP_DELETE, copyAction);
		}

//		if(pasteAction.updateSelection(selection))
		pasteAction.selectionChanged(selection);
		if(pasteAction.isApplicableForSelection())
		{
			menu.appendToGroup(INavigatorContribution.GROUP_DELETE, pasteAction);
		}
		
//		moveAction.selectionChanged(selection);
//		if(moveAction.isApplicableForSelection())
//		{
//			menu.appendToGroup(INavigatorContribution.GROUP_DELETE, moveAction);
//		}

		deleteResourceAction.setEnabled(false);		
		if(resSelection.size() == selection.size())
		{
			deleteResourceAction.selectionChanged(resSelection);
			menu.appendToGroup(INavigatorContribution.GROUP_DELETE, deleteResourceAction);
			
			addBookmarkAction.selectionChanged(resSelection);
			if(addBookmarkAction.isEnabled())
				menu.appendToGroup(INavigatorContribution.GROUP_DELETE, addBookmarkAction);
		}
		if(!deleteResourceAction.isEnabled())
		{
			menu.add(new Separator());
			deleteChildEObject.selectionChanged(selection);
			if(deleteChildEObject.isEnabled())
				menu.add(deleteChildEObject);
		}
		renameAction.selectionChanged(selection);
		if(renameAction.isApplicableForSelection())
		{
			menu.appendToGroup(INavigatorContribution.GROUP_DELETE, renameAction);
		}

		if(selection.size() == 1)
		{
			propertyDialogAction.selectionChanged(selection);
			if(propertyDialogAction.isApplicableForSelection())
				menu.appendToGroup(INavigatorContribution.GROUP_PROPERTIES, propertyDialogAction);
		}

	}
		
	/**
	 * @see org.eclipse.ui.actions.ActionGroup#fillActionBars(org.eclipse.ui.IActionBars)
	 */
	public void fillActionBars(IActionBars actionBars)
	{
		actionBars.setGlobalActionHandler(ActionFactory.PROPERTIES.getId(),	propertyDialogAction);
		actionBars.setGlobalActionHandler(IDEActionFactory.BOOKMARK.getId(), addBookmarkAction);
		actionBars.setGlobalActionHandler(IDEActionFactory.ADD_TASK.getId(), addTaskAction);
		actionBars.setGlobalActionHandler(ActionFactory.DELETE.getId(), deleteResourceAction);

		frameListActionGroup.fillActionBars(actionBars);
		newActionGroup.fillActionBars(actionBars);
		workingSetFilterActionGroup.fillActionBars(actionBars);
		sorterActionGroup.fillActionBars(actionBars);
		
		super.fillActionBars(actionBars);

		IMenuManager menu = actionBars.getMenuManager();
		menu.add(new Separator());
		menu.add(toggleLinkingAction);
		//menu.add(toggleFolderAction);
		menu.add(toggleEObjectChildren);
		
		IToolBarManager toolBar = actionBars.getToolBarManager();
		toolBar.add(new Separator());
		toolBar.add(collapseAllAction);		
		toolBar.add(toggleLinkingAction);
		//toolBar.add(toggleFolderAction);
	}
	
	/**
	 * @see org.eclipse.ui.actions.ActionGroup#updateActionBars()
	 */
	public void updateActionBars()
	{
		TestNavigator testNavigator = (TestNavigator)getNavigator();
		Object currentFrameInput = ((TreeFrame)testNavigator.getFrameList().getCurrentFrame()).getInput();
		toggleFolderAction.setEnabled((currentFrameInput == null) || (testNavigator.getInitialViewerInput() == currentFrameInput));
		toggleEObjectChildren.setEnabled((currentFrameInput == null) || (!(currentFrameInput instanceof EObject)));
		
		IStructuredSelection structuredSelection = (IStructuredSelection) getContext().getSelection();
		IStructuredSelection resourceStructuredSelection = getResourceSelection(structuredSelection);
		
		propertyDialogAction.setEnabled(structuredSelection.size() == 1);
		addBookmarkAction.selectionChanged(resourceStructuredSelection);
		deleteResourceAction.selectionChanged(resourceStructuredSelection);
		addTaskAction.selectionChanged(structuredSelection);	
		refreshAction.selectionChanged(structuredSelection);
		openProjectAction.selectionChanged(structuredSelection);
		closeProjectAction.selectionChanged(structuredSelection);
		copyAction.selectionChanged(structuredSelection);
		pasteAction.selectionChanged(structuredSelection);
		renameAction.selectionChanged(structuredSelection);
	
		frameListActionGroup.updateActionBars();
		sorterActionGroup.updateActionBars();
		newActionGroup.updateActionBars();
		workingSetFilterActionGroup.updateActionBars();
	}
	
	/**
	 * @see org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup#runDefaultAction(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void runDefaultAction(IStructuredSelection selection)
	{
		openAction.selectionChanged(selection);
		openAction.run();
	}
	
	/**
	 * @see org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup#handleKeyPressed(org.eclipse.swt.events.KeyEvent)
	 */
	public void handleKeyPressed(KeyEvent event)
	{
		if(event.character == SWT.DEL && (event.stateMask == SWT.None))
		{
			if(deleteResourceAction.isEnabled())
				deleteResourceAction.run();
			//- Swallow the event.
			event.doit = false;
		}
		
		if(event.keyCode == SWT.F5 && event.stateMask == SWT.None) {
			if(refreshAction.isEnabled()) {
				refreshAction.run();
			}
			//- Swallow the event.
			event.doit = false;
		}
		
		if(event.keyCode == SWT.F2 && event.stateMask == SWT.None) {
			if(renameAction.isEnabled()) {
				renameAction.run();
			}
			//- Swallow the event.
			event.doit = false;
		}

		super.handleKeyPressed(event);
	}

	/**
	 * Returns an structured selection with resource elements based on the given
	 * structured selection.
	 * @param structuredSelection
	 * @return IStructuredSelection
	 * @see TestNavigator#getFile(Object)
	 */
	protected IStructuredSelection getResourceSelection(IStructuredSelection structuredSelection)
	{
		TestNavigator testNavigator = (TestNavigator)getNavigator();
		List resources = new ArrayList();
		for(Iterator i = structuredSelection.iterator(); i.hasNext();)
		{
			Object object = i.next();
			if((object instanceof IResource) && (((IResource)object).getType() != IResource.FILE))
			{
				resources.add(object);
			}
			else
			{
				IFile file = testNavigator.getFile(object);
				if(file != null)
					resources.add(file);				
			}			
		}
		return new StructuredSelection(resources);
	}

	/**
	 * Returns the image descriptor from the eclipse UI plugin with the given relative 
	 * path.
	 * @return ImageDescriptor;
	 */
	protected ImageDescriptor getImageDescriptor(String relativePath)
	{
		String iconPath = "icons/full/"; //$NON-NLS-1$
		try
		{
			URL installURL = Platform.getBundle(PlatformUI.PLUGIN_ID).getEntry("/");
			URL url = new URL(installURL, iconPath + relativePath);
			return ImageDescriptor.createFromURL(url);
		}
		catch(MalformedURLException e)
		{
			return ImageDescriptor.getMissingImageDescriptor();
		}
	}

	/** returns the rename action defined for test navigator
	 * @return rename action
	 */
	public IAction getRenameAction() {
		return renameAction;
	}

	/** retruns the copy action defined for the test navigator view
	 * @return copy action
	 */
	public IAction getCopyAction() {
		return copyAction;
	}

	/** retruns the paste action defined for the test navigator view
	 * @return paste action
	 */
	public IAction getPasteAction() {
		return pasteAction;
	}

	/** retruns the refresh action defined for the test navigator view
	 * @return refresh action
	 */
	public IAction getRefreshAction() {
		return refreshAction;
	}	
}
